Package = require './package'
_       = require 'underscore'
coffee  = require 'coffee-script'
ejs     = require 'ejs'
fs      = require 'fs'
path    = require 'path'

template = fs.readFileSync(path.join __dirname, 'template.ejs.js').toString().replace(/\n/g, '')

###
Serves the provided string of JavaScript source to as
the result of request object provided.
###
serveJavaScript = (res, source='')->
  res.statusCode = 200
  res.setHeader 'Content-type', 'text/javascript'
  res.end source

###
Returns a 500 error
###
error = (err, res)->
  console.error err
  res.statusCode = 500
  res.end err

###
Compiles the provided CoffeeScript source and send it as the
response.
###
serveCoffeeScript = (res, source)->
  try
    source = coffee.compile source
    serveJavaScript res, source
  catch e
    error e, res

###
Loads the source at the given path.
###
loadSource = (confPath, source, res)->
  fs.readFile path.join(path.dirname(confPath), source), (err, data)->
    return error err, res if err
    data = data.toString()
    serve = if source.match new RegExp '\.coffee$' then serveCoffeeScript else serveJavaScript
    serve res, data

###
A function generator:
Creates a middleware function for serving jspackle packages in 'development'
mode.
###
module.exports = (confPath, urlPath)->

  configs =
    url: urlPath

  package = JSON.parse fs.readFileSync confPath
  p = new Package root: '', path: confPath

  ###
  Creates a main JavaScript file which synchronously loads all the
  development sources in order via XHR
  ###
  _.extend configs, package, sources: p.sources
  main = ejs.render template, configs

  (req, res, next)->
    if req.url is urlPath
      serveJavaScript res, main
    else if req.url.match new RegExp "^#{urlPath}"
      loadSource confPath, req.url.replace(urlPath, ''), res
    else
      next()
